##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Simple, importable content classes.
"""

from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from zope.interface import implementer

from ..interfaces import ICSVAware
from ..interfaces import IDAVAware
from ..interfaces import IINIAware


class TestSimpleItem(SimpleItem):
    pass


class TestSimpleItemWithProperties(SimpleItem, PropertyManager):
    pass


KNOWN_CSV = """\
one,two,three
four,five,six
"""


@implementer(ICSVAware)
class TestCSVAware(SimpleItem):
    _was_put = None
    _csv = KNOWN_CSV

    def as_csv(self):
        return self._csv

    def put_csv(self, text):
        self._was_put = text


KNOWN_INI = """\
[DEFAULT]
title = %s
description = %s
"""


@implementer(IINIAware)
class TestINIAware(SimpleItem):
    _was_put = None
    title = 'INI title'
    description = 'INI description'

    def as_ini(self):
        return KNOWN_INI % (self.title, self.description)

    def put_ini(self, text):
        self._was_put = text


KNOWN_DAV = b"""\
Title: %s
Description: %s

%s
"""


@implementer(IDAVAware)
class TestDAVAware(SimpleItem):
    _was_put = None
    title = b'DAV title'
    description = b'DAV description'
    body = b'DAV body'

    def manage_FTPget(self):
        return KNOWN_DAV % (self.title, self.description, self.body)

    def PUT(self, REQUEST, RESPONSE):
        self._was_put = REQUEST.get('BODY', '')
        stream = REQUEST.get('BODYFILE', None)
        self._was_put_as_read = stream.read()



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""PageTemplate export / import support unit tests.
"""

import unittest

from ...testing import BodyAdapterTestCase
from ...testing import ExportImportZCMLLayer


_PAGETEMPLATE_BODY = b"""\
<html>
  <div>Foo</div>
</html>
"""


class ZopePageTemplateBodyAdapterTests(BodyAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import ZopePageTemplateBodyAdapter

        return ZopePageTemplateBodyAdapter

    def _populate(self, obj):
        obj.write(_PAGETEMPLATE_BODY)

    def setUp(self):
        from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

        self._obj = ZopePageTemplate('foo_template')
        self._BODY = _PAGETEMPLATE_BODY


def test_suite():
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromTestCase(
            ZopePageTemplateBodyAdapterTests),
    ))



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Component registry export / import support unit tests.
"""

import unittest

from AccessControl.class_init import InitializeClass
from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import aq_base
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Products.Five.component import enableSite
from Products.Five.component.interfaces import IObjectManagerSite
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import handle
from zope.component import queryAdapter
from zope.component import queryUtility
from zope.component import subscribers
from zope.component.globalregistry import base
from zope.component.hooks import clearSite
from zope.component.hooks import setHooks
from zope.component.hooks import setSite
from zope.interface import Interface
from zope.interface import implementer

from ..interfaces import IBody
from ..interfaces import IComponentsHandlerBlacklist
from ..testing import BodyAdapterTestCase
from ..testing import DummySetupEnviron
from ..testing import ExportImportZCMLLayer
from ..tests.common import DummyImportContext


try:
    from five.localsitemanager.registry import PersistentComponents
except ModuleNotFoundError:
    # Avoid generating a spurious dependency
    PersistentComponents = None


def createComponentRegistry(context):
    enableSite(context, iface=IObjectManagerSite)

    components = PersistentComponents('++etc++site')
    components.__bases__ = (base, )
    components.__parent__ = aq_base(context)
    # Make sure calls to getSiteManager on me return myself
    # necessary because OFS.ObjectManager.getSiteManager expects _components
    components._components = components
    context.setSiteManager(components)


class IDummyInterface(Interface):
    """A dummy interface."""

    def verify():
        """Returns True."""


class IDummyInterface2(Interface):
    """A second dummy interface."""

    def verify():
        """Returns True."""


@implementer(IDummyInterface)
class DummyUtility:
    """A dummy utility."""

    def verify(self):
        return True


class IAnotherDummy(Interface):
    """A third dummy interface."""

    def inc():
        """Increments handle count"""


class IAnotherDummy2(Interface):
    """A second dummy interface."""

    def verify():
        """Returns True."""


@implementer(IAnotherDummy)
class DummyObject:
    """A dummy object to pass to the handler."""

    handled = 0

    def inc(self):
        self.handled += 1


@implementer(IAnotherDummy2)
class DummyAdapter:
    """A dummy adapter."""

    def __init__(self, context):
        pass

    def verify(self):
        return True


def dummy_handler(context):
    """A dummy event handler."""

    context.inc()


@implementer(IDummyInterface)
class DummyTool(SimpleItem):
    """A dummy tool."""

    id = 'dummy_tool'
    meta_type = 'dummy tool'
    security = ClassSecurityInfo()

    @security.public
    def verify(self):
        return True


InitializeClass(DummyTool)


@implementer(IDummyInterface2)
class DummyTool2(SimpleItem):
    """A second dummy tool."""

    id = 'dummy_tool2'
    meta_type = 'dummy tool2'
    security = ClassSecurityInfo()

    @security.public
    def verify(self):
        return True


InitializeClass(DummyTool2)


@implementer(IComponentsHandlerBlacklist)
class DummyBlacklist:
    """A blacklist."""

    def getExcludedInterfaces(self):
        return (IDummyInterface, )


_COMPONENTS_BODY = b"""\
<?xml version="1.0" encoding="utf-8"?>
<componentregistry>
 <adapters>
  <adapter factory="Products.GenericSetup.tests.test_components.DummyAdapter"
     for="zope.interface.Interface"
     provides="Products.GenericSetup.tests.test_components.IAnotherDummy2"/>
  <adapter name="foo"
     factory="Products.GenericSetup.tests.test_components.DummyAdapter"
     for="zope.interface.Interface"
     provides="Products.GenericSetup.tests.test_components.IAnotherDummy2"/>
 </adapters>
 <subscribers>
  <subscriber
     factory="Products.GenericSetup.tests.test_components.DummyAdapter"
     for="Products.GenericSetup.tests.test_components.IAnotherDummy"
     provides="Products.GenericSetup.tests.test_components.IAnotherDummy2"/>
  <subscriber for="Products.GenericSetup.tests.test_components.IAnotherDummy"
     handler="Products.GenericSetup.tests.test_components.dummy_handler"/>
 </subscribers>
 <utilities>
  <utility factory="Products.GenericSetup.tests.test_components.DummyUtility"
     id="dummy_utility"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface"/>
  <utility name="dummy tool name"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface"
     object="dummy_tool"/>
  <utility name="dummy tool name2"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface2"
     object="dummy_tool2"/>
  <utility name="foo"
     factory="Products.GenericSetup.tests.test_components.DummyUtility"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface2"/>
 </utilities>
</componentregistry>
"""

_REMOVE_IMPORT = b"""\
<?xml version="1.0" encoding="utf-8"?>
<componentregistry>
 <adapters>
  <adapter factory="Products.GenericSetup.tests.test_components.DummyAdapter"
     provides="Products.GenericSetup.tests.test_components.IAnotherDummy2"
     for="*" remove="True"/>
 </adapters>
 <subscribers>
  <subscriber
     factory="Products.GenericSetup.tests.test_components.DummyAdapter"
     for="Products.GenericSetup.tests.test_components.IAnotherDummy"
     provides="Products.GenericSetup.tests.test_components.IAnotherDummy2"
     remove="True"/>
  <subscriber
     for="Products.GenericSetup.tests.test_components.IAnotherDummy"
     handler="Products.GenericSetup.tests.test_components.dummy_handler"
     remove="True"/>
 </subscribers>
 <utilities>
  <utility id="dummy_utility"
     factory="Products.GenericSetup.tests.test_components.DummyUtility"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface"
     remove="True"/>
  <utility name="dummy tool name"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface"
     object="dummy_tool" remove="True"/>
  <utility name="foo"
     factory="Products.GenericSetup.tests.test_components.DummyUtility"
     interface="Products.GenericSetup.tests.test_components.IDummyInterface2"
     remove="True"/>
 </utilities>
</componentregistry>
"""

_INTERFACE_COMPONENT = b"""\
<?xml version="1.0" encoding="utf-8"?>
<componentregistry>
 <adapters/>
 <subscribers/>
 <utilities>
  <utility name="test_interface"
     component="Products.GenericSetup.tests.test_components.ITestInterface"
     interface="Products.GenericSetup.tests.test_components.ITestInterfaceType"/>
 </utilities>
</componentregistry>
"""


class ComponentRegistryXMLAdapterTests(BodyAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from Products.GenericSetup.components import \
            ComponentRegistryXMLAdapter
        return ComponentRegistryXMLAdapter

    def _populate(self, obj):
        obj.registerAdapter(DummyAdapter, required=(None, ))
        obj.registerAdapter(DummyAdapter, required=(None, ), name='foo')

        obj.registerSubscriptionAdapter(DummyAdapter,
                                        required=(IAnotherDummy, ))
        obj.registerHandler(dummy_handler, required=(IAnotherDummy, ))

        util = DummyUtility()
        name = 'dummy_utility'
        util.__name__ = name
        util.__parent__ = aq_base(obj)
        obj._setObject(name,
                       aq_base(util),
                       set_owner=False,
                       suppress_events=True)
        obj.registerUtility(aq_base(obj[name]), IDummyInterface)

        util = DummyUtility()
        name = ('Products.GenericSetup.tests.test_components.'
                'IDummyInterface2-foo')
        util.__name__ = name
        util.__parent__ = aq_base(obj)
        obj._setObject(name,
                       aq_base(util),
                       set_owner=False,
                       suppress_events=True)
        obj.registerUtility(aq_base(obj[name]), IDummyInterface2, name='foo')

        tool = aq_base(obj.aq_parent['dummy_tool'])
        obj.registerUtility(tool, IDummyInterface, name='dummy tool name')

        tool2 = aq_base(obj.aq_parent['dummy_tool2'])
        obj.registerUtility(tool2, IDummyInterface2, name='dummy tool name2')

    def _verifyImport(self, obj):
        adapted = queryAdapter(object(), IAnotherDummy2)
        self.assertTrue(IAnotherDummy2.providedBy(adapted))
        self.assertTrue(adapted.verify())

        adapted = queryAdapter(object(), IAnotherDummy2, name='foo')
        self.assertTrue(IAnotherDummy2.providedBy(adapted))
        self.assertTrue(adapted.verify())

        dummy = DummyObject()
        results = [
            adap.verify() for adap in subscribers([dummy], IAnotherDummy2)
        ]
        self.assertEqual(results, [True])

        dummy = DummyObject()
        handle(dummy)
        self.assertEqual(dummy.handled, 1)

        util = queryUtility(IDummyInterface2, name='foo')
        self.assertTrue(IDummyInterface.providedBy(util))
        self.assertTrue(util.verify())
        self.assertEqual(util.__parent__, obj)
        name = ('Products.GenericSetup.tests.test_components.'
                'IDummyInterface2-foo')
        self.assertEqual(util.__name__, name)
        self.assertIn(name, obj.objectIds())

        util = queryUtility(IDummyInterface)
        self.assertTrue(IDummyInterface.providedBy(util))
        self.assertTrue(util.verify())
        self.assertEqual(util.__parent__, obj)
        name = 'dummy_utility'
        self.assertEqual(util.__name__, name)
        self.assertIn(name, obj.objectIds())

        util = queryUtility(IDummyInterface, name='dummy tool name')
        self.assertTrue(IDummyInterface.providedBy(util))
        self.assertTrue(util.verify())
        self.assertEqual(util.meta_type, 'dummy tool')

        # make sure we can get the tool by normal means
        tool = getattr(obj.aq_parent, 'dummy_tool')
        self.assertEqual(tool.meta_type, 'dummy tool')
        self.assertEqual(repr(aq_base(util)), repr(aq_base(tool)))

        util = queryUtility(IDummyInterface2, name='dummy tool name2')
        self.assertTrue(IDummyInterface2.providedBy(util))
        self.assertTrue(util.verify())
        self.assertEqual(util.meta_type, 'dummy tool2')

        # make sure we can get the tool by normal means
        tool = getattr(obj.aq_parent, 'dummy_tool2')
        self.assertEqual(tool.meta_type, 'dummy tool2')
        self.assertEqual(repr(aq_base(util)), repr(aq_base(tool)))

    def test_blacklist_get(self):
        obj = self._obj
        self._populate(obj)

        # Register our blacklist
        gsm = getGlobalSiteManager()
        gsm.registerUtility(DummyBlacklist(),
                            IComponentsHandlerBlacklist,
                            name='dummy')

        context = DummySetupEnviron()
        adapted = getMultiAdapter((obj, context), IBody)

        body = adapted.body
        self.assertNotIn(b'IComponentsHandlerBlacklist', body)
        self.assertNotIn(b'test_components.IDummyInterface"', body)

    def test_blacklist_set(self):
        obj = self._obj
        # Register our blacklist
        gsm = getGlobalSiteManager()
        gsm.registerUtility(DummyBlacklist(),
                            IComponentsHandlerBlacklist,
                            name='dummy')

        context = DummySetupEnviron()
        adapted = getMultiAdapter((obj, context), IBody)
        adapted.body = self._BODY

        util = queryUtility(IDummyInterface2, name='foo')
        self.assertTrue(IDummyInterface.providedBy(util))
        self.assertTrue(util.verify())
        util = queryUtility(IDummyInterface)
        self.assertIsNone(util)

        # now in update mode
        context._should_purge = False
        adapted = getMultiAdapter((obj, context), IBody)
        adapted.body = self._BODY

        util = queryUtility(IDummyInterface2, name='foo')
        self.assertTrue(IDummyInterface.providedBy(util))
        self.assertTrue(util.verify())
        util = queryUtility(IDummyInterface)
        self.assertIsNone(util)

        # and again in update mode
        adapted = getMultiAdapter((obj, context), IBody)
        adapted.body = self._BODY

        util = queryUtility(IDummyInterface2, name='foo')
        self.assertTrue(IDummyInterface.providedBy(util))
        self.assertTrue(util.verify())
        util = queryUtility(IDummyInterface)
        self.assertIsNone(util)

    def test_remove_components(self):
        from ..components import importComponentRegistry

        obj = self._obj
        self._populate(obj)
        self._verifyImport(obj)

        context = DummyImportContext(obj, False)
        context._files['componentregistry.xml'] = _REMOVE_IMPORT
        importComponentRegistry(context)

        adapted = queryAdapter(object(), IAnotherDummy2)
        self.assertIsNone(adapted)

        # This one should still exist
        adapted = queryAdapter(object(), IAnotherDummy2, name='foo')
        self.assertIsNotNone(adapted)

        dummy = DummyObject()
        results = [
            adap.verify() for adap in subscribers([dummy], IAnotherDummy2)
        ]
        self.assertEqual(results, [])

        dummy = DummyObject()
        handle(dummy)
        self.assertEqual(dummy.handled, 0)

        util = queryUtility(IDummyInterface2, name='foo')
        name = ('Products.GenericSetup.tests.test_components.'
                'IDummyInterface2-foo')
        self.assertIsNone(util)
        self.assertNotIn(name, obj.objectIds())

        util = queryUtility(IDummyInterface)
        self.assertIsNone(util)
        self.assertNotIn('dummy_utility', obj.objectIds())

        util = queryUtility(IDummyInterface, name='dummy tool name')
        self.assertIsNone(util)

        # This one should still exist
        util = queryUtility(IDummyInterface2, name='dummy tool name2')
        self.assertIsNotNone(util)

    def test_export_interface_component(self):
        sm = self._obj
        sm.registerUtility(ITestInterface, ITestInterfaceType,
                           "test_interface")
        context = DummySetupEnviron()
        adapted = getMultiAdapter((sm, context), IBody)
        body = adapted.body
        self.assertEqual(body, _INTERFACE_COMPONENT)

    def test_import_interface_component(self):
        from ..components import importComponentRegistry
        sm = self._obj
        context = DummyImportContext(sm, False)
        context._files['componentregistry.xml'] = _INTERFACE_COMPONENT
        importComponentRegistry(context)
        self.assertIs(
            sm.queryUtility(ITestInterfaceType, name="test_interface"),
            ITestInterface)

    def setUp(self):
        # Create and enable a local component registry
        site = Folder()
        createComponentRegistry(site)
        setHooks()
        setSite(site)
        sm = getSiteManager()

        tool = DummyTool()
        site._setObject(tool.id, tool)

        tool2 = DummyTool2()
        site._setObject(tool2.id, tool2)

        self._obj = sm
        self._BODY = _COMPONENTS_BODY

    def tearDown(self):
        clearSite()
        # Make sure our global utility is gone again
        gsm = getGlobalSiteManager()
        gsm.unregisterUtility(provided=IComponentsHandlerBlacklist,
                              name='dummy')


class ITestInterface(Interface):
    """interface registered as utility."""


class ITestInterfaceType(Interface):
    """interface provided by ``ITestInterface``."""


if PersistentComponents is not None:

    def test_suite():
        # reimport to make sure tests are run from Products
        from ..tests.test_components import ComponentRegistryXMLAdapterTests

        return unittest.TestSuite(
            (unittest.defaultTestLoader.loadTestsFromTestCase(
                ComponentRegistryXMLAdapterTests), ))
else:

    def test_suite():
        return unittest.TestSuite()



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""ZCTextIndex export / import support unit tests.
"""

import unittest

from Acquisition import Implicit

from ...testing import ExportImportZCMLLayer
from ...testing import NodeAdapterTestCase


_PLEXICON_XML = b"""\
<object name="foo_plexicon" meta_type="ZCTextIndex Lexicon">
 <element name="Whitespace splitter" group="Word Splitter"/>
 <element name="Case Normalizer" group="Case Normalizer"/>
 <element name="Remove listed stop words only" group="Stop Words"/>
</object>
"""

_ZCTEXT_XML = b"""\
<index name="foo_zctext" meta_type="ZCTextIndex">
 <indexed_attr value="foo_zctext"/>
 <indexed_attr value="baz_zctext"/>
 <extra name="index_type" value="Okapi BM25 Rank"/>
 <extra name="lexicon_id" value="foo_plexicon"/>
</index>
"""


class _extra:

    pass


class DummyCatalog(Implicit):

    pass


class ZCLexiconNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import ZCLexiconNodeAdapter

        return ZCLexiconNodeAdapter

    def _populate(self, obj):
        from Products.ZCTextIndex.Lexicon import CaseNormalizer
        from Products.ZCTextIndex.Lexicon import Splitter
        from Products.ZCTextIndex.Lexicon import StopWordRemover
        obj._pipeline = (Splitter(), CaseNormalizer(), StopWordRemover())

    def setUp(self):
        from Products.ZCTextIndex.ZCTextIndex import PLexicon

        self._obj = PLexicon('foo_plexicon')
        self._XML = _PLEXICON_XML


class ZCTextIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import ZCTextIndexNodeAdapter

        return ZCTextIndexNodeAdapter

    def _populate(self, obj):
        obj._indexed_attrs = ['foo_zctext', 'baz_zctext']

    def setUp(self):
        from Products.ZCTextIndex.ZCTextIndex import PLexicon
        from Products.ZCTextIndex.ZCTextIndex import ZCTextIndex

        catalog = DummyCatalog()
        catalog.foo_plexicon = PLexicon('foo_plexicon')
        extra = _extra()
        extra.lexicon_id = 'foo_plexicon'
        extra.index_type = 'Okapi BM25 Rank'
        self._obj = ZCTextIndex('foo_zctext', extra=extra,
                                caller=catalog).__of__(catalog)
        self._XML = _ZCTEXT_XML


class UnchangedTests(unittest.TestCase):

    layer = ExportImportZCMLLayer

    def test_ZCLexicon(self):
        from xml.dom.minidom import parseString

        from Products.ZCTextIndex.PipelineFactory import element_factory

        from ...testing import DummySetupEnviron
        from ..exportimport import ZCLexiconNodeAdapter

        _XML = b"""\
        <object name="foo_plexicon" meta_type="ZCTextIndex Lexicon">
        <element name="foo" group="gs"/>
        <element name="bar" group="gs"/>
        </object>
        """
        environ = DummySetupEnviron()
        _before = object(), object(), object()

        class DummyLexicon:
            _wids, _words, length = _before

        lex = DummyLexicon()
        lex._pipeline = foo, bar = object(), object()
        adapted = ZCLexiconNodeAdapter(lex, environ)
        element_factory._groups['gs'] = {'foo': lambda: foo,
                                         'bar': lambda: bar}
        try:
            adapted.node = parseString(_XML).documentElement  # no raise
        finally:
            del element_factory._groups['gs']
        self.assertIs(lex._wids, _before[0])
        self.assertIs(lex._words, _before[1])
        self.assertIs(lex.length, _before[2])

    def test_ZCTextIndex(self):
        from xml.dom.minidom import parseString

        from Products.ZCTextIndex.ZCTextIndex import PLexicon
        from Products.ZCTextIndex.ZCTextIndex import ZCTextIndex

        from ...testing import DummySetupEnviron
        from ..exportimport import ZCTextIndexNodeAdapter
        _XML = b"""\
        <index name="foo_zctext" meta_type="ZCTextIndex">
        <indexed_attr value="bar"/>
        <extra name="index_type" value="Okapi BM25 Rank"/>
        <extra name="lexicon_id" value="foo_plexicon"/>
        </index>
        """
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        catalog = DummyCatalog()
        catalog.foo_plexicon = PLexicon('foo_plexicon')
        extra = _extra()
        extra.lexicon_id = 'foo_plexicon'
        extra.index_type = 'Okapi BM25 Rank'
        index = ZCTextIndex('foo_field', extra=extra, field_name='bar',
                            caller=catalog).__of__(catalog)
        index.clear = _no_clear
        adapted = ZCTextIndexNodeAdapter(index, environ)
        adapted.node = parseString(_XML).documentElement  # no raise


def test_suite():
    loader = unittest.defaultTestLoader
    return unittest.TestSuite((
        loader.loadTestsFromTestCase(ZCLexiconNodeAdapterTests),
        loader.loadTestsFromTestCase(ZCTextIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(UnchangedTests),
    ))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" GenericSetup:  Role-permission export / import
"""

from AccessControl.class_init import InitializeClass
from AccessControl.Permission import Permission
from AccessControl.SecurityInfo import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from .permissions import ManagePortal
from .utils import CONVERTER
from .utils import DEFAULT
from .utils import KEY
from .utils import ExportConfiguratorBase
from .utils import ImportConfiguratorBase
from .utils import _xmldir


#
#   Configurator entry points
#
_FILENAME = 'rolemap.xml'


def importRolemap(context):
    """ Import roles / permission map from an XML file.

    o 'context' must implement IImportContext.

    o Register via Python:

      registry = site.setup_tool.setup_steps
      registry.registerStep('importRolemap', '20040518-01',
                            Products.GenericSetup.rolemap.importRolemap,
                            (), 'Role / Permission import',
                            'Import roles and map roles to permissions')

    o Register via XML:

      <setup-step id="importRolemap"
                  version="20040518-01"
                  handler="Products.GenericSetup.rolemap.importRolemap"
                  title="Role / Permission import"
      >Import additional roles, and map roles to permissions.</setup-step>

    """
    site = context.getSite()
    encoding = context.getEncoding()
    logger = context.getLogger('rolemap')

    text = context.readDataFile(_FILENAME)

    if text is not None:

        if context.shouldPurge():

            items = list(site.__dict__.items())

            for k, v in items:  # WAAA

                if k == '__ac_roles__':
                    delattr(site, k)

                if k.startswith('_') and k.endswith('_Permission'):
                    delattr(site, k)

        rc = RolemapImportConfigurator(site, encoding)
        rolemap_info = rc.parseXML(text)

        immediate_roles = list(getattr(site, '__ac_roles__', []))
        already = {}

        for role in site.valid_roles():
            already[role] = 1

        for role in rolemap_info['roles']:

            if already.get(role) is None:
                immediate_roles.append(role)
                already[role] = 1

        immediate_roles.sort()
        site.__ac_roles__ = tuple(immediate_roles)

        for permission in rolemap_info['permissions']:

            site.manage_permission(permission['name'],
                                   permission.get('roles', []),
                                   permission['acquire'])

    logger.info('Role / permission map imported.')


def exportRolemap(context):
    """ Export roles / permission map as an XML file

    o 'context' must implement IExportContext.

    o Register via Python:

      registry = site.setup_tool.export_steps
      registry.registerStep('exportRolemap',
                            Products.GenericSetup.rolemap.exportRolemap
                            'Role / Permission export',
                            'Export roles and role / permission map')

    o Register via XML:

      <export-script id="exportRolemap"
                     version="20040518-01"
                     handler="Products.GenericSetup.rolemap.exportRolemap"
                     title="Role / Permission export"
      >Export additional roles, and role / permission map.</export-script>

    """
    site = context.getSite()
    logger = context.getLogger('rolemap')

    rc = RolemapExportConfigurator(site).__of__(site)
    text = rc.generateXML().encode('utf-8')

    context.writeDataFile(_FILENAME, text, 'text/xml')

    logger.info('Role / permission map exported.')


class RolemapExportConfigurator(ExportConfiguratorBase):

    """ Synthesize XML description of sitewide role-permission settings.
    """
    security = ClassSecurityInfo()

    @security.protected(ManagePortal)
    def listRoles(self):
        """ List the valid role IDs for our site.
        """
        return self._site.valid_roles()

    @security.protected(ManagePortal)
    def listPermissions(self):
        """ List permissions for export.

        o Returns a sqeuence of mappings describing locally-modified
          permission / role settings.  Keys include:

          'permission' -- the name of the permission

          'acquire' -- a flag indicating whether to acquire roles from the
              site's container

          'roles' -- the list of roles which have the permission.

        o Do not include permissions which both acquire and which define
          no local changes to the acquired policy.
        """
        permissions = []
        valid_roles = self.listRoles()

        for perm in self._site.ac_inherited_permissions(1):

            name = perm[0]
            p = Permission(name, perm[1], self._site)
            roles = p.getRoles(default=[])
            acquire = isinstance(roles, list)  # tuple means don't acquire
            roles = [r for r in roles if r in valid_roles]
            roles.sort()

            if roles or not acquire:
                permissions.append({'name': name, 'acquire': acquire,
                                    'roles': roles})

        return permissions

    def _getExportTemplate(self):

        return PageTemplateFile('rmeExport.xml', _xmldir)


InitializeClass(RolemapExportConfigurator)


class RolemapImportConfigurator(ImportConfiguratorBase):

    """ Synthesize XML description of sitewide role-permission settings.
    """
    security = ClassSecurityInfo()

    def _getImportMapping(self):

        return {
            'rolemap': {
                'roles': {CONVERTER: self._convertToUnique, DEFAULT: ()},
                'permissions': {CONVERTER: self._convertToUnique}},
            'roles': {'role': {KEY: None}},
            'role': {'name': {KEY: None}},
            'permissions': {'permission': {KEY: None, DEFAULT: ()}},
            'permission': {'name': {},
                           'role': {KEY: 'roles'},
                           'acquire': {CONVERTER: self._convertToBoolean}}}


InitializeClass(RolemapImportConfigurator)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""PluginIndexes export / import support unit tests.
"""

import unittest

from ...testing import ExportImportZCMLLayer
from ...testing import NodeAdapterTestCase


_DATE_XML = b"""\
<index name="foo_date" meta_type="DateIndex">
 <property name="index_naive_time_as_local">True</property>
 <property name="precision">0</property>
</index>
"""

_DATERANGE_XML = b"""\
<index name="foo_daterange" meta_type="DateRangeIndex" since_field="bar"
   until_field="baz"/>
"""

_FIELD_XML = b"""\
<index name="foo_field" meta_type="FieldIndex">
 <indexed_attr value="bar"/>
</index>
"""

_KEYWORD_XML = b"""\
<index name="foo_keyword" meta_type="KeywordIndex">
 <indexed_attr value="bar"/>
</index>
"""

_ODDBALL_XML = b"""\
<index name="foo_keyword" meta_type="OddballIndex">
</index>
"""

_PATH_XML = b"""\
<index name="foo_path" meta_type="PathIndex"/>
"""

_SET_XML = b"""\
<filtered_set name="bar" meta_type="PythonFilteredSet" expression="True"/>
"""

_TOPIC_XML = b"""\
<index name="foo_topic" meta_type="TopicIndex">
 <filtered_set name="bar" meta_type="PythonFilteredSet" expression="True"/>
 <filtered_set name="baz" meta_type="PythonFilteredSet" expression="False"/>
</index>
"""


class DateIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import DateIndexNodeAdapter
        return DateIndexNodeAdapter

    def setUp(self):
        from Products.PluginIndexes.DateIndex.DateIndex import DateIndex
        self._obj = DateIndex('foo_date')
        self._obj._setPropValue('precision', 0)
        self._XML = _DATE_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'foo_date')


class DateRangeIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import DateRangeIndexNodeAdapter
        return DateRangeIndexNodeAdapter

    def _populate(self, obj):
        obj._edit('bar', 'baz')

    def setUp(self):
        from Products.PluginIndexes.DateRangeIndex.DateRangeIndex import \
            DateRangeIndex
        self._obj = DateRangeIndex('foo_daterange')
        self._XML = _DATERANGE_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'foo_daterange')
        self.assertEqual(obj._since_field, 'bar')
        self.assertEqual(obj._until_field, 'baz')


class FieldIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import PluggableIndexNodeAdapter
        return PluggableIndexNodeAdapter

    def _populate(self, obj):
        obj.indexed_attrs = ('bar',)

    def setUp(self):
        from Products.PluginIndexes.FieldIndex.FieldIndex import FieldIndex
        self._obj = FieldIndex('foo_field')
        self._XML = _FIELD_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'foo_field')
        self.assertEqual(obj.indexed_attrs, ['bar'])


class KeywordIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import PluggableIndexNodeAdapter
        return PluggableIndexNodeAdapter

    def _populate(self, obj):
        obj.indexed_attrs = ('bar',)

    def setUp(self):
        from Products.PluginIndexes.KeywordIndex.KeywordIndex import \
            KeywordIndex
        self._obj = KeywordIndex('foo_keyword')
        self._XML = _KEYWORD_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'foo_keyword')
        self.assertEqual(obj.indexed_attrs, ['bar'])


class PathIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import PathIndexNodeAdapter
        return PathIndexNodeAdapter

    def setUp(self):
        from Products.PluginIndexes.PathIndex.PathIndex import PathIndex
        self._obj = PathIndex('foo_path')
        self._XML = _PATH_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'foo_path')


class FilteredSetNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import FilteredSetNodeAdapter
        return FilteredSetNodeAdapter

    def _populate(self, obj):
        obj.setExpression('True')

    def setUp(self):
        from Products.PluginIndexes.TopicIndex.FilteredSet import \
            PythonFilteredSet
        self._obj = PythonFilteredSet('bar', '')
        self._XML = _SET_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'bar')
        self.assertEqual(obj.getExpression(), 'True')


class TopicIndexNodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from ..exportimport import TopicIndexNodeAdapter
        return TopicIndexNodeAdapter

    def _populate(self, obj):
        obj.addFilteredSet('bar', 'PythonFilteredSet', 'True')
        obj.addFilteredSet('baz', 'PythonFilteredSet', 'False')

    def setUp(self):
        from Products.PluginIndexes.TopicIndex.TopicIndex import TopicIndex
        self._obj = TopicIndex('foo_topic')
        self._XML = _TOPIC_XML

    def _verifyImport(self, obj):
        self.assertEqual(obj.id, 'foo_topic')
        self.assertEqual(len(obj.filteredSets), 2)


class UnchangedTests(unittest.TestCase):

    layer = ExportImportZCMLLayer

    def test_FieldIndex(self):
        from xml.dom.minidom import parseString

        from Products.PluginIndexes.FieldIndex.FieldIndex import FieldIndex

        from ...testing import DummySetupEnviron
        from ..exportimport import PluggableIndexNodeAdapter
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        index = FieldIndex('foo_field')
        index.indexed_attrs = ['bar']
        index.clear = _no_clear
        adapted = PluggableIndexNodeAdapter(index, environ)
        adapted.node = parseString(_FIELD_XML).documentElement  # no raise

    def test_KeywordIndex(self):
        from xml.dom.minidom import parseString

        from Products.PluginIndexes.KeywordIndex.KeywordIndex import \
            KeywordIndex

        from ...testing import DummySetupEnviron
        from ..exportimport import PluggableIndexNodeAdapter
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        index = KeywordIndex('foo_keyword')
        index.indexed_attrs = ['bar']
        index.clear = _no_clear
        adapted = PluggableIndexNodeAdapter(index, environ)
        adapted.node = parseString(_KEYWORD_XML).documentElement  # no raise

    def test_OddballIndex(self):
        # Some indexes, e.g. Plone's 'GopipIndex', use ths adapter but don't
        # have 'indexed_attrs'.
        from xml.dom.minidom import parseString

        from ...testing import DummySetupEnviron
        from ..exportimport import PluggableIndexNodeAdapter

        class Oddball:
            def clear(*a):
                raise AssertionError("Don't clear me!")

        index = Oddball()
        environ = DummySetupEnviron()
        adapted = PluggableIndexNodeAdapter(index, environ)
        adapted.node = parseString(_ODDBALL_XML).documentElement  # no raise

    def test_DateIndex(self):
        from xml.dom.minidom import parseString

        from Products.PluginIndexes.DateIndex.DateIndex import DateIndex

        from ...testing import DummySetupEnviron
        from ..exportimport import DateIndexNodeAdapter
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        index = DateIndex('foo_date')
        index._setPropValue('index_naive_time_as_local', True)
        index._setPropValue('precision', 0)
        index.clear = _no_clear
        adapted = DateIndexNodeAdapter(index, environ)
        adapted.node = parseString(_DATE_XML).documentElement  # no raise

    def test_DateRangeIndex(self):
        from xml.dom.minidom import parseString

        from Products.PluginIndexes.DateRangeIndex.DateRangeIndex import \
            DateRangeIndex

        from ...testing import DummySetupEnviron
        from ..exportimport import DateRangeIndexNodeAdapter
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        index = DateRangeIndex('foo_daterange')
        index._since_field = 'bar'
        index._until_field = 'baz'
        index.clear = _no_clear
        adapted = DateRangeIndexNodeAdapter(index, environ)
        adapted.node = parseString(_DATERANGE_XML).documentElement  # no raise

    def test_FilteredSet(self):
        from xml.dom.minidom import parseString

        from Products.PluginIndexes.TopicIndex.FilteredSet import \
            PythonFilteredSet

        from ...testing import DummySetupEnviron
        from ..exportimport import FilteredSetNodeAdapter
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        index = PythonFilteredSet('bar', 'True')
        index.clear = _no_clear
        adapted = FilteredSetNodeAdapter(index, environ)
        adapted.node = parseString(_SET_XML).documentElement  # no raise

    def test_TopicIndex(self):
        from xml.dom.minidom import parseString

        from Products.GenericSetup.testing import DummySetupEnviron
        from Products.PluginIndexes.TopicIndex.TopicIndex import TopicIndex

        from ..exportimport import TopicIndexNodeAdapter
        environ = DummySetupEnviron()

        def _no_clear(*a):
            raise AssertionError("Don't clear me!")

        index = TopicIndex('topics')
        index.addFilteredSet('bar', 'PythonFilteredSet', 'True')
        index.addFilteredSet('baz', 'PythonFilteredSet', 'False')
        bar = index.filteredSets['bar']
        baz = index.filteredSets['baz']
        bar.clear = baz.clear = _no_clear
        adapted = TopicIndexNodeAdapter(index, environ)
        adapted.node = parseString(_SET_XML).documentElement  # no raise


def test_suite():
    loader = unittest.defaultTestLoader
    return unittest.TestSuite((
        loader.loadTestsFromTestCase(DateIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(DateRangeIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(FieldIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(KeywordIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(PathIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(FilteredSetNodeAdapterTests),
        loader.loadTestsFromTestCase(TopicIndexNodeAdapterTests),
        loader.loadTestsFromTestCase(UnchangedTests),
    ))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Registry unit tests.
"""

import unittest

from OFS.Folder import Folder
from zope.interface import Interface

from .. import EXTENSION
from ..testing import ExportImportZCMLLayer
from .common import BaseRegistryTests
from .conformance import ConformsToIExportStepRegistry
from .conformance import ConformsToIImportStepRegistry
from .conformance import ConformsToIProfileRegistry
from .conformance import ConformsToIStepRegistry
from .conformance import ConformsToIToolsetRegistry


###############################
#   Dummy handlers
###############################
def ONE_FUNC(context): pass


def TWO_FUNC(context): pass


def THREE_FUNC(context): pass


def FOUR_FUNC(context): pass


def DOC_FUNC(site):
    """This is the first line.

    This is the second line.
    """


ONE_FUNC_NAME = f'{__name__}.{ONE_FUNC.__name__}'
TWO_FUNC_NAME = f'{__name__}.{TWO_FUNC.__name__}'
THREE_FUNC_NAME = f'{__name__}.{THREE_FUNC.__name__}'
FOUR_FUNC_NAME = f'{__name__}.{FOUR_FUNC.__name__}'
DOC_FUNC_NAME = f'{__name__}.{DOC_FUNC.__name__}'

###############################
#   SSR tests
###############################


class ImportStepRegistryTests(BaseRegistryTests, ConformsToIStepRegistry,
                              ConformsToIImportStepRegistry):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):

        from ..registry import ImportStepRegistry
        return ImportStepRegistry

    def test_empty(self):

        registry = self._makeOne()

        self.assertEqual(len(registry.listSteps()), 0)
        self.assertEqual(len(registry.listStepMetadata()), 0)
        self.assertEqual(len(registry.sortSteps()), 0)

    def test_getStep_nonesuch(self):

        registry = self._makeOne()

        self.assertEqual(registry.getStep('nonesuch'), None)
        self.assertEqual(registry.getStep('nonesuch'), None)
        default = object()
        self.assertIs(
            registry.getStepMetadata(
                'nonesuch', default),
            default
        )
        self.assertIs(registry.getStep('nonesuch', default), default)
        self.assertIs(
            registry.getStepMetadata(
                'nonesuch', default),
            default
        )

    def test_getStep_defaulted(self):

        registry = self._makeOne()
        default = object()

        self.assertIs(registry.getStep('nonesuch', default), default)
        self.assertEqual(registry.getStepMetadata('nonesuch', {}), {})

    def test_registerStep_docstring(self):

        registry = self._makeOne()

        registry.registerStep(id='docstring', version='1', handler=DOC_FUNC,
                              dependencies=())

        info = registry.getStepMetadata('docstring')
        self.assertEqual(info['id'], 'docstring')
        self.assertEqual(info['handler'], DOC_FUNC_NAME)
        self.assertEqual(info['dependencies'], ())
        self.assertEqual(info['title'], 'This is the first line.')
        self.assertEqual(info['description'], 'This is the second line.')

    def test_registerStep_docstring_override(self):

        registry = self._makeOne()

        registry.registerStep(id='docstring', version='1', handler=DOC_FUNC,
                              dependencies=(), title='Title')

        info = registry.getStepMetadata('docstring')
        self.assertEqual(info['id'], 'docstring')
        self.assertEqual(info['handler'], DOC_FUNC_NAME)
        self.assertEqual(info['dependencies'], ())
        self.assertEqual(info['title'], 'Title')
        self.assertEqual(info['description'], 'This is the second line.')

    def test_registerStep_single(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two', 'three'),
                              title='One Step', description='One small step')

        steps = registry.listSteps()
        self.assertEqual(len(steps), 1)
        self.assertIn('one', steps)
        self.assertEqual(registry.getStep('one'), ONE_FUNC)

        info = registry.getStepMetadata('one')
        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['version'], '1')
        self.assertEqual(info['handler'], ONE_FUNC_NAME)
        self.assertEqual(info['dependencies'], ('two', 'three'))
        self.assertEqual(info['title'], 'One Step')
        self.assertEqual(info['description'], 'One small step')

        info_list = registry.listStepMetadata()
        self.assertEqual(len(info_list), 1)
        self.assertEqual(info, info_list[0])

    def test_registerStep_conflict(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC_NAME)

        self.assertRaises(KeyError, registry.registerStep, id='one',
                          version='0', handler=ONE_FUNC)

        registry.registerStep(id='one', version='1', handler=ONE_FUNC_NAME)

        info_list = registry.listStepMetadata()
        self.assertEqual(len(info_list), 1)

    def test_registerStep_replacement(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two', 'three'),
                              title='One Step', description='One small step')

        registry.registerStep(id='one', version='1.1', handler=ONE_FUNC,
                              dependencies=(), title='Leads to Another',
                              description='Another small step')

        info = registry.getStepMetadata('one')
        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['version'], '1.1')
        self.assertEqual(info['dependencies'], ())
        self.assertEqual(info['title'], 'Leads to Another')
        self.assertEqual(info['description'], 'Another small step')

    def test_registerStep_multiple(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=())

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=())

        registry.registerStep(id='three', version='3', handler=THREE_FUNC,
                              dependencies=())

        steps = registry.listSteps()
        self.assertEqual(len(steps), 3)
        self.assertIn('one', steps)
        self.assertIn('two', steps)
        self.assertIn('three', steps)

    def test_sortStep_simple(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two',))

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=())

        steps = registry.sortSteps()
        self.assertEqual(len(steps), 2)
        one = steps.index('one')
        two = steps.index('two')

        self.assertTrue(0 <= two < one)

    def test_sortStep_chained(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('three',), title='One small step')

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=('one',), title='Texas two step')

        registry.registerStep(id='three', version='3', handler=THREE_FUNC,
                              dependencies=(), title='Gimme three steps')

        steps = registry.sortSteps()
        self.assertEqual(len(steps), 3)
        one = steps.index('one')
        two = steps.index('two')
        three = steps.index('three')

        self.assertTrue(0 <= three < one < two)

    def test_sortStep_complex(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two',), title='One small step')

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=('four',), title='Texas two step')

        registry.registerStep(id='three', version='3', handler=THREE_FUNC,
                              dependencies=('four',),
                              title='Gimme three steps')

        registry.registerStep(id='four', version='4', handler=FOUR_FUNC,
                              dependencies=(), title='Four step program')

        steps = registry.sortSteps()
        self.assertEqual(len(steps), 4)
        one = steps.index('one')
        two = steps.index('two')
        three = steps.index('three')
        four = steps.index('four')

        self.assertTrue(0 <= four < two < one)
        self.assertTrue(0 <= four < three)

    def test_sortStep_equivalence(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two', 'three'),
                              title='One small step')

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=('four',), title='Texas two step')

        registry.registerStep(id='three', version='3', handler=THREE_FUNC,
                              dependencies=('four',),
                              title='Gimme three steps')

        registry.registerStep(id='four', version='4', handler=FOUR_FUNC,
                              dependencies=(), title='Four step program')

        steps = registry.sortSteps()
        self.assertEqual(len(steps), 4)
        one = steps.index('one')
        two = steps.index('two')
        three = steps.index('three')
        four = steps.index('four')

        self.assertTrue(0 <= four < two < one)
        self.assertTrue(0 <= four < three < one)

    def test_checkComplete_simple(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two',))

        incomplete = registry.checkComplete()
        self.assertEqual(len(incomplete), 1)
        self.assertIn(('one', 'two'), incomplete)

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=())

        self.assertEqual(len(registry.checkComplete()), 0)

    def test_checkComplete_double(self):

        registry = self._makeOne()

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two', 'three'))

        incomplete = registry.checkComplete()
        self.assertEqual(len(incomplete), 2)
        self.assertIn(('one', 'two'), incomplete)
        self.assertIn(('one', 'three'), incomplete)

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=())

        incomplete = registry.checkComplete()
        self.assertEqual(list(incomplete), [('one', 'three')])

        registry.registerStep(id='three', version='3', handler=THREE_FUNC,
                              dependencies=())

        self.assertEqual(list(registry.checkComplete()), [])

        registry.registerStep(id='two', version='2.1', handler=TWO_FUNC,
                              dependencies=('four',))

        incomplete = registry.checkComplete()
        self.assertIn(('two', 'four'), incomplete)

    def test_generateXML_empty(self):

        registry = self._makeOne().__of__(self.app)

        self._compareDOM(registry.generateXML(), _EMPTY_IMPORT_XML, debug=True)

    def test_generateXML_single(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=(), title='One Step',
                              description='One small step')

        self._compareDOM(registry.generateXML(), _SINGLE_IMPORT_XML)

    def test_generateXML_ordered(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=('two',), title='One Step',
                              description='One small step')

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=('three',), title='Two Steps',
                              description='Texas two step')

        registry.registerStep(id='three', version='3', handler=THREE_FUNC,
                              dependencies=(), title='Three Steps',
                              description='Gimme three steps')

        self._compareDOM(registry.generateXML(), _ORDERED_IMPORT_XML)

    def test_parseXML_empty(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='one', version='1', handler=ONE_FUNC,
                              dependencies=(), description='One small step')

        info_list = registry.parseXML(_EMPTY_IMPORT_XML)

        self.assertEqual(len(info_list), 0)

    def test_parseXML_single(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='two', version='2', handler=TWO_FUNC,
                              dependencies=(), title='Two Steps',
                              description='Texas two step')

        info_list = registry.parseXML(_SINGLE_IMPORT_XML)

        self.assertEqual(len(info_list), 1)

        info = info_list[0]
        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['version'], '1')
        self.assertEqual(info['handler'], ONE_FUNC_NAME)
        self.assertEqual(info['dependencies'], ())
        self.assertEqual(info['title'], 'One Step')
        self.assertIn('One small step', info['description'])


_EMPTY_IMPORT_XML = """\
<?xml version="1.0"?>
<import-steps>
</import-steps>
"""

_SINGLE_IMPORT_XML = """\
<?xml version="1.0"?>
<import-steps>
 <import-step id="one"
             version="1"
             handler="%s"
             title="One Step">
  One small step
 </import-step>
</import-steps>
""" % ONE_FUNC_NAME

_ORDERED_IMPORT_XML = f"""\
<?xml version="1.0"?>
<import-steps>
 <import-step id="one"
             version="1"
             handler="{ONE_FUNC_NAME}"
             title="One Step">
  <dependency step="two" />
  One small step
 </import-step>
 <import-step id="three"
             version="3"
             handler="{THREE_FUNC_NAME}"
             title="Three Steps">
  Gimme three steps
 </import-step>
 <import-step id="two"
             version="2"
             handler="{TWO_FUNC_NAME}"
             title="Two Steps">
  <dependency step="three" />
  Texas two step
 </import-step>
</import-steps>
"""

###############################
#   ESR tests
###############################


class ExportStepRegistryTests(BaseRegistryTests, ConformsToIStepRegistry,
                              ConformsToIExportStepRegistry):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):

        from ..registry import ExportStepRegistry
        return ExportStepRegistry

    def _makeOne(self, *args, **kw):

        return self._getTargetClass()(*args, **kw)

    def test_empty(self):

        registry = self._makeOne()
        self.assertEqual(len(registry.listSteps()), 0)
        self.assertEqual(len(registry.listStepMetadata()), 0)

    def test_getStep_nonesuch(self):

        registry = self._makeOne()
        self.assertEqual(registry.getStep('nonesuch'), None)

    def test_getStep_defaulted(self):
        def default(x):
            return False

        registry = self._makeOne()
        self.assertEqual(registry.getStep('nonesuch', default), default)

    def test_getStepMetadata_nonesuch(self):

        registry = self._makeOne()
        self.assertEqual(registry.getStepMetadata('nonesuch'), None)

    def test_getStepMetadata_defaulted(self):

        registry = self._makeOne()
        self.assertEqual(registry.getStepMetadata('nonesuch', {}), {})

    def test_registerStep_simple(self):

        registry = self._makeOne()
        registry.registerStep('one', ONE_FUNC_NAME)
        info = registry.getStepMetadata('one', {})

        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['handler'], ONE_FUNC_NAME)
        self.assertEqual(info['title'], 'one')
        self.assertEqual(info['description'], '')

    def test_registerStep_docstring(self):

        registry = self._makeOne()
        registry.registerStep('one', DOC_FUNC)
        info = registry.getStepMetadata('one', {})

        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['handler'], DOC_FUNC_NAME)
        self.assertEqual(info['title'], 'This is the first line.')
        self.assertEqual(info['description'], 'This is the second line.')

    def test_registerStep_docstring_with_override(self):

        registry = self._makeOne()
        registry.registerStep('one', DOC_FUNC, description='Description')
        info = registry.getStepMetadata('one', {})

        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['handler'], DOC_FUNC_NAME)
        self.assertEqual(info['title'], 'This is the first line.')
        self.assertEqual(info['description'], 'Description')

    def test_registerStep_collision(self):

        registry = self._makeOne()
        registry.registerStep('one', ONE_FUNC)
        registry.registerStep('one', TWO_FUNC)
        info = registry.getStepMetadata('one', {})

        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['handler'], TWO_FUNC_NAME)
        self.assertEqual(info['title'], 'one')
        self.assertEqual(info['description'], '')

    def test_generateXML_empty(self):

        registry = self._makeOne().__of__(self.app)

        self._compareDOM(registry.generateXML(), _EMPTY_EXPORT_XML)

    def test_generateXML_single(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='one', handler=ONE_FUNC, title='One Step',
                              description='One small step')

        self._compareDOM(registry.generateXML(), _SINGLE_EXPORT_XML)

    def test_generateXML_ordered(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='one', handler=ONE_FUNC, title='One Step',
                              description='One small step')

        registry.registerStep(id='two', handler=TWO_FUNC, title='Two Steps',
                              description='Texas two step')

        registry.registerStep(id='three', handler=THREE_FUNC,
                              title='Three Steps',
                              description='Gimme three steps')

        self._compareDOM(registry.generateXML(), _ORDERED_EXPORT_XML)

    def test_parseXML_empty(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='one', handler=ONE_FUNC,
                              description='One small step')

        info_list = registry.parseXML(_EMPTY_EXPORT_XML)

        self.assertEqual(len(info_list), 0)

    def test_parseXML_single(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='two', handler=TWO_FUNC, title='Two Steps',
                              description='Texas two step')

        info_list = registry.parseXML(_SINGLE_EXPORT_XML)

        self.assertEqual(len(info_list), 1)

        info = info_list[0]
        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['handler'], ONE_FUNC_NAME)
        self.assertEqual(info['title'], 'One Step')
        self.assertIn('One small step', info['description'])

    def test_parseXML_single_as_ascii(self):

        registry = self._makeOne().__of__(self.app)

        registry.registerStep(id='two', handler=TWO_FUNC, title='Two Steps',
                              description='Texas two step')

        info_list = registry.parseXML(_SINGLE_EXPORT_XML, encoding='ascii')

        self.assertEqual(len(info_list), 1)

        info = info_list[0]
        self.assertEqual(info['id'], 'one')
        self.assertEqual(info['handler'], ONE_FUNC_NAME)
        self.assertEqual(info['title'], 'One Step')
        self.assertIn('One small step', info['description'])


_EMPTY_EXPORT_XML = """\
<?xml version="1.0"?>
<export-steps>
</export-steps>
"""

_SINGLE_EXPORT_XML = """\
<?xml version="1.0"?>
<export-steps>
 <export-step id="one"
                handler="%s"
                title="One Step">
  One small step
 </export-step>
</export-steps>
""" % ONE_FUNC_NAME

_ORDERED_EXPORT_XML = f"""\
<?xml version="1.0"?>
<export-steps>
 <export-step id="one"
                handler="{ONE_FUNC_NAME}"
                title="One Step">
  One small step
 </export-step>
 <export-step id="three"
                handler="{THREE_FUNC_NAME}"
                title="Three Steps">
  Gimme three steps
 </export-step>
 <export-step id="two"
                handler="{TWO_FUNC_NAME}"
                title="Two Steps">
  Texas two step
 </export-step>
</export-steps>
"""


###############################
#   ToolsetRegistry tests
###############################


class ToolsetRegistryTests(BaseRegistryTests, ConformsToIToolsetRegistry):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):

        from ..registry import ToolsetRegistry
        return ToolsetRegistry

    def _initSite(self):

        self.app.site = Folder(id='site')
        site = self.app.site

        return site

    def test_empty(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        self.assertEqual(len(configurator.listForbiddenTools()), 0)
        self.assertEqual(len(configurator.listRequiredTools()), 0)
        self.assertEqual(len(configurator.listRequiredToolInfo()), 0)

        self.assertRaises(
            KeyError, configurator.getRequiredToolInfo, 'nonesuch')

    def test_addForbiddenTool_multiple(self):

        VERBOTTEN = ('foo', 'bar', 'bam')

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        for verbotten in VERBOTTEN:
            configurator.addForbiddenTool(verbotten)

        self.assertEqual(
            len(configurator.listForbiddenTools()), len(VERBOTTEN))

        for verbotten in configurator.listForbiddenTools():
            self.assertIn(verbotten, VERBOTTEN)

    def test_addForbiddenTool_duplicate(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.addForbiddenTool('once')
        configurator.addForbiddenTool('once')

        self.assertEqual(len(configurator.listForbiddenTools()), 1)

    def test_addForbiddenTool_but_required(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.addRequiredTool('required', 'some.dotted.name')
        self.assertEqual(len(configurator.listRequiredToolInfo()), 1)
        self.assertEqual(len(configurator.listForbiddenTools()), 0)

        configurator.addForbiddenTool('required')
        self.assertEqual(len(configurator.listRequiredToolInfo()), 0)
        self.assertEqual(len(configurator.listForbiddenTools()), 1)

    def test_addRequiredTool_multiple(self):

        REQUIRED = (
            ('one', 'path.to.one'),
            ('two', 'path.to.two'),
            ('three', 'path.to.three'),
        )

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        for tool_id, dotted_name in REQUIRED:
            configurator.addRequiredTool(tool_id, dotted_name)

        self.assertEqual(len(configurator.listRequiredTools()), len(REQUIRED))

        for id in [x[0] for x in REQUIRED]:
            self.assertIn(id, configurator.listRequiredTools())

        self.assertEqual(
            len(configurator.listRequiredToolInfo()), len(REQUIRED))

        for tool_id, dotted_name in REQUIRED:
            info = configurator.getRequiredToolInfo(tool_id)
            self.assertEqual(info['id'], tool_id)
            self.assertEqual(info['class'], dotted_name)

    def test_addRequiredTool_duplicate(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.addRequiredTool('required', 'some.dotted.name')
        configurator.addRequiredTool('required', 'another.name')

        info = configurator.getRequiredToolInfo('required')
        self.assertEqual(info['id'], 'required')
        self.assertEqual(info['class'], 'another.name')

    def test_addRequiredTool_but_forbidden(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.addForbiddenTool('forbidden')
        self.assertEqual(len(configurator.listRequiredToolInfo()), 0)
        self.assertEqual(len(configurator.listForbiddenTools()), 1)

        configurator.addRequiredTool('forbidden', 'a.name')
        self.assertEqual(len(configurator.listRequiredToolInfo()), 1)
        self.assertEqual(len(configurator.listForbiddenTools()), 0)

    def test_generateXML_empty(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        self._compareDOM(configurator.generateXML(), _EMPTY_TOOLSET_XML)

    def test_generateXML_normal(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.addForbiddenTool('doomed')
        configurator.addRequiredTool('mandatory', 'path.to.one')
        configurator.addRequiredTool('obligatory', 'path.to.another')

        configurator.parseXML(_NORMAL_TOOLSET_XML)

    def test_parseXML_empty(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.parseXML(_EMPTY_TOOLSET_XML)

        self.assertEqual(len(configurator.listForbiddenTools()), 0)
        self.assertEqual(len(configurator.listRequiredTools()), 0)

    def test_parseXML_normal(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.parseXML(_NORMAL_TOOLSET_XML)

        self.assertEqual(len(configurator.listForbiddenTools()), 1)
        self.assertIn('doomed', configurator.listForbiddenTools())

        self.assertEqual(len(configurator.listRequiredTools()), 2)

        self.assertIn('mandatory', configurator.listRequiredTools())
        info = configurator.getRequiredToolInfo('mandatory')
        self.assertEqual(info['class'], 'path.to.one')

        self.assertIn('obligatory', configurator.listRequiredTools())
        info = configurator.getRequiredToolInfo('obligatory')
        self.assertEqual(info['class'], 'path.to.another')

    def test_parseXML_confused(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        configurator.parseXML(_CONFUSED_TOOLSET_XML)
        self.assertEqual(len(configurator.listForbiddenTools()), 0)
        self.assertEqual(len(configurator.listRequiredToolInfo()), 1)

    def test_parseXML_wrong(self):

        site = self._initSite()
        configurator = self._makeOne().__of__(site)

        # The 'remove' keyword is explicitly not supported.
        self.assertRaises(
            ValueError, configurator.parseXML, _WRONG_TOOLSET_XML)


_EMPTY_TOOLSET_XML = """\
<?xml version="1.0"?>
<tool-setup>
</tool-setup>
"""

_NORMAL_TOOLSET_XML = """\
<?xml version="1.0"?>
<tool-setup>
 <forbidden tool_id="doomed" />
 <required tool_id="mandatory" class="path.to.one" />
 <required tool_id="obligatory" class="path.to.another" />
</tool-setup>
"""

_CONFUSED_TOOLSET_XML = """\
<?xml version="1.0"?>
<tool-setup>
 <forbidden tool_id="confused" />
 <required tool_id="confused" class="path.to.one" />
</tool-setup>
"""

_WRONG_TOOLSET_XML = """\
<?xml version="1.0"?>
<tool-setup>
 <required tool_id="confused" class="path.to.one" remove="" />
</tool-setup>
"""


class ISite(Interface):
    pass


class IDerivedSite(ISite):
    pass


class IAnotherSite(Interface):
    pass


class ProfileRegistryTests(BaseRegistryTests, ConformsToIProfileRegistry):

    def _getTargetClass(self):

        from ..registry import ProfileRegistry
        return ProfileRegistry

    def test_empty(self):

        registry = self._makeOne()

        self.assertEqual(len(registry.listProfiles()), 0)
        self.assertEqual(len(registry.listProfiles()), 0)
        self.assertRaises(KeyError, registry.getProfileInfo, 'nonesuch')

    def test_registerProfile_normal(self):

        NAME = 'one'
        TITLE = 'One'
        DESCRIPTION = 'One profile'
        PATH = '/path/to/one'
        PRODUCT = 'TestProduct'
        PROFILE_TYPE = EXTENSION
        PROFILE_ID = 'TestProduct:one'

        registry = self._makeOne()
        registry.registerProfile(NAME, TITLE, DESCRIPTION, PATH, PRODUCT,
                                 PROFILE_TYPE)

        self.assertEqual(len(registry.listProfiles()), 1)
        self.assertEqual(len(registry.listProfileInfo()), 1)

        info = registry.getProfileInfo(PROFILE_ID)

        self.assertEqual(info['id'], PROFILE_ID)
        self.assertEqual(info['title'], TITLE)
        self.assertEqual(info['description'], DESCRIPTION)
        self.assertEqual(info['path'], PATH)
        self.assertEqual(info['product'], PRODUCT)
        self.assertEqual(info['type'], PROFILE_TYPE)
        self.assertEqual(info['for'], None)
        self.assertEqual(info['pre_handler'], None)
        self.assertEqual(info['post_handler'], None)

        # We strip off any 'profile-' or 'snapshot-' at the beginning
        # of the profile id.
        info2 = registry.getProfileInfo('profile-' + PROFILE_ID)
        self.assertEqual(info, info2)
        info3 = registry.getProfileInfo('snapshot-' + PROFILE_ID)
        self.assertEqual(info, info3)

    def test_registerProfile_duplicate_same(self):

        NAME = 'one'
        TITLE = 'One'
        DESCRIPTION = 'One profile'
        PATH = '/path/to/one'

        registry = self._makeOne()
        registry.registerProfile(NAME, TITLE, DESCRIPTION, PATH)
        # If we register the exact same profile, this should be fine.
        # We will just keep the old one.
        info = registry.getProfileInfo('other:one')
        registry.registerProfile(NAME, TITLE, DESCRIPTION, PATH)
        new_info = registry.getProfileInfo('other:one')
        self.assertEqual(info, new_info)

    def test_registerProfile_duplicate_other(self):

        NAME = 'one'
        TITLE = 'One'
        DESCRIPTION = 'One profile'
        PATH = '/path/to/one'

        registry = self._makeOne()
        registry.registerProfile(NAME, TITLE, DESCRIPTION, PATH)
        DESCRIPTION = 'Other profile'
        self.assertRaises(KeyError, registry.registerProfile,
                          NAME, TITLE, DESCRIPTION, PATH)

    def test_registerProfile_site_type(self):

        NAME = 'one'
        TITLE = 'One'
        DESCRIPTION = 'One profile'
        PATH = '/path/to/one'
        PRODUCT = 'TestProduct'
        PROFILE_ID = 'TestProduct:one'
        PROFILE_TYPE = EXTENSION
        FOR = ISite
        NOT_FOR = IAnotherSite
        DERIVED_FOR = IDerivedSite

        registry = self._makeOne()
        registry.registerProfile(NAME, TITLE, DESCRIPTION, PATH, PRODUCT,
                                 PROFILE_TYPE, for_=FOR)

        self.assertEqual(len(registry.listProfiles()), 1)
        self.assertEqual(len(registry.listProfiles(for_=FOR)), 1)
        self.assertEqual(len(registry.listProfiles(for_=DERIVED_FOR)), 1)
        self.assertEqual(len(registry.listProfiles(for_=NOT_FOR)), 0)

        self.assertEqual(len(registry.listProfileInfo()), 1)
        self.assertEqual(len(registry.listProfileInfo(for_=FOR)), 1)
        self.assertEqual(len(registry.listProfileInfo(for_=DERIVED_FOR)), 1)
        self.assertEqual(len(registry.listProfileInfo(for_=NOT_FOR)), 0)

        # Verify that these lookups succeed...
        info1 = registry.getProfileInfo(PROFILE_ID)
        info2 = registry.getProfileInfo(PROFILE_ID, for_=FOR)
        info3 = registry.getProfileInfo(PROFILE_ID, for_=DERIVED_FOR)

        self.assertEqual(info1, info2)
        self.assertEqual(info1, info3)

        # ...and that this one fails.
        self.assertRaises(KeyError, registry.getProfileInfo, PROFILE_ID,
                          for_=NOT_FOR)

        info = registry.getProfileInfo(PROFILE_ID, for_=FOR)

        self.assertEqual(info['id'], PROFILE_ID)
        self.assertEqual(info['title'], TITLE)
        self.assertEqual(info['description'], DESCRIPTION)
        self.assertEqual(info['path'], PATH)
        self.assertEqual(info['product'], PRODUCT)
        self.assertEqual(info['type'], PROFILE_TYPE)
        self.assertEqual(info['for'], FOR)

    def test_getProfileInfo_tarball(self):
        # When importing a tarball, some code calls getProfileInfo with id
        # None.  This must not crash.
        registry = self._makeOne()
        self.assertEqual(registry.getProfileInfo(None),
                         {'description': '',
                          'for': None,
                          'id': '',
                          'path': '',
                          'product': '',
                          'title': '',
                         'type': None})


def test_suite():
    loader = unittest.defaultTestLoader
    return unittest.TestSuite((
        loader.loadTestsFromTestCase(ImportStepRegistryTests),
        loader.loadTestsFromTestCase(ExportStepRegistryTests),
        loader.loadTestsFromTestCase(ToolsetRegistryTests),
        loader.loadTestsFromTestCase(ProfileRegistryTests),
    ))



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" GenericSetup profile metadata
"""

import os
from xml.parsers.expat import ExpatError

from .utils import CONVERTER
from .utils import DEFAULT
from .utils import KEY
from .utils import ImportConfiguratorBase
from .utils import _getProductPath


METADATA_XML = 'metadata.xml'


class ProfileMetadata(ImportConfiguratorBase):

    """ Extracts profile metadata from metadata.xml file.
    """

    def __init__(self, path, encoding=None, product=None):

        # don't call the base class __init__ b/c we don't have (or need)
        # a site

        self._path = path
        if product is not None:
            # Handle relative paths
            try:
                product_path = _getProductPath(product)
            except ValueError:
                pass
            else:
                self._path = os.path.join(product_path, path)

        self._encoding = encoding

    def __call__(self):

        full_path = os.path.join(self._path, METADATA_XML)
        if not os.path.exists(full_path):
            return {}

        with open(full_path) as fp:
            try:
                return self.parseXML(fp.read())
            except ExpatError as e:
                raise ExpatError(f'{full_path}: {e}')

    def _getImportMapping(self):

        return {
            'metadata':
                {'description': {CONVERTER: self._convertToUnique},
                 'version': {CONVERTER: self._convertToUnique},
                 'dependencies': {CONVERTER: self._convertToUnique}},
            'description': {'#text': {KEY: None, DEFAULT: ''}},
            'version': {'#text': {KEY: None}},
            'dependencies': {'dependency': {KEY: None, DEFAULT: ()}},
            'dependency': {'#text': {KEY: None}}}



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Node adapter testing utils.
"""

from xml.dom.minidom import parseString

from OFS.interfaces import IItem
from Testing.ZopeTestCase.layer import ZopeLite
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface.verify import verifyClass
from zope.testing.cleanup import cleanUp

from Products.GenericSetup.interfaces import IBody
from Products.GenericSetup.interfaces import INode
from Products.GenericSetup.interfaces import ISetupEnviron


class DummyLogger:

    def __init__(self, id, messages):
        self._id = id
        self._messages = messages

    def info(self, msg, *args, **kwargs):
        self._messages.append((20, self._id, msg))

    def warning(self, msg, *args, **kwargs):
        self._messages.append((30, self._id, msg))


@implementer(ISetupEnviron)
class DummySetupEnviron:

    """Context for body im- and exporter.
    """

    def __init__(self):
        self._notes = []
        self._should_purge = True

    def getLogger(self, name):
        return DummyLogger(name, self._notes)

    def shouldPurge(self):
        return self._should_purge


class IDummyMarker(IItem):

    pass


class _AdapterTestCaseBase:

    def _populate(self, obj):
        pass

    def _verifyImport(self, obj):
        pass


class BodyAdapterTestCase(_AdapterTestCaseBase):

    def test_z3interfaces(self):
        verifyClass(IBody, self._getTargetClass())

    def test_body_get(self):
        self._populate(self._obj)
        context = DummySetupEnviron()
        adapted = getMultiAdapter((self._obj, context), IBody)
        self.assertEqual(adapted.body, self._BODY)
        self.assertIsInstance(adapted.body, bytes)

    def test_body_set(self):
        context = DummySetupEnviron()
        adapted = getMultiAdapter((self._obj, context), IBody)
        adapted.body = self._BODY
        self._verifyImport(self._obj)
        self.assertEqual(adapted.body, self._BODY)

        # now in update mode
        context._should_purge = False
        adapted = getMultiAdapter((self._obj, context), IBody)
        adapted.body = self._BODY
        self._verifyImport(self._obj)
        self.assertEqual(adapted.body, self._BODY)

        # and again in update mode
        adapted = getMultiAdapter((self._obj, context), IBody)
        adapted.body = self._BODY
        self._verifyImport(self._obj)
        self.assertEqual(adapted.body, self._BODY)


class NodeAdapterTestCase(_AdapterTestCaseBase):

    def test_z3interfaces(self):
        verifyClass(INode, self._getTargetClass())

    def test_node_get(self):
        self._populate(self._obj)
        context = DummySetupEnviron()
        adapted = getMultiAdapter((self._obj, context), INode)
        self.assertEqual(adapted.node.toprettyxml(' ', encoding='utf-8'),
                         self._XML)

    def test_node_set(self):
        context = DummySetupEnviron()
        adapted = getMultiAdapter((self._obj, context), INode)
        adapted.node = parseString(self._XML).documentElement
        self._verifyImport(self._obj)
        self.assertEqual(adapted.node.toprettyxml(' ', encoding='utf-8'),
                         self._XML)

        # now in update mode
        context._should_purge = False
        adapted = getMultiAdapter((self._obj, context), INode)
        adapted.node = parseString(self._XML).documentElement
        self._verifyImport(self._obj)
        self.assertEqual(adapted.node.toprettyxml(' ', encoding='utf-8'),
                         self._XML)

        # and again in update mode
        adapted = getMultiAdapter((self._obj, context), INode)
        adapted.node = parseString(self._XML).documentElement
        self._verifyImport(self._obj)
        self.assertEqual(adapted.node.toprettyxml(' ', encoding='utf-8'),
                         self._XML)


class ExportImportZCMLLayer(ZopeLite):

    @classmethod
    def setUp(cls):
        import AccessControl
        import Products.Five
        import Zope2.App
        import zope.traversing
        from Zope2.App import zcml

        import Products.GenericSetup

        zcml.load_config('meta.zcml', Zope2.App)
        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('meta.zcml', Products.GenericSetup)
        zcml.load_config('permissions.zcml', AccessControl)
        zcml.load_config('configure.zcml', zope.traversing)
        zcml.load_config('permissions.zcml', Products.Five)
        zcml.load_config('configure.zcml', Products.GenericSetup)

    @classmethod
    def tearDown(cls):
        cleanUp()
